<?php
require_once 'config.php';

// Define lock file location
define('LOCK_FILE', '/home/ssgenius/public_html/mautic-data-fetcher/tmp/fetch_bounce_by_webhook_events.lock');

/**
 * Check if the script is already running, and create a lock file.
 */
function checkAndCreateLock() {
    if (file_exists(LOCK_FILE)) {
        $pid = file_get_contents(LOCK_FILE);
        if (posix_kill((int)$pid, 0)) {
            die("⚠️ Script is already running (PID: $pid). Exiting...\n");
        } else {
            // If PID no longer exists, assume it's a stale lock and remove it.
            unlink(LOCK_FILE);
        }
    }

    // Create a new lock file with the current PID.
    file_put_contents(LOCK_FILE, getmypid());
}

/**
 * Remove the lock file when script finishes.
 */
function removeLock() {
    if (file_exists(LOCK_FILE)) {
        unlink(LOCK_FILE);
    }
}

/**
 * Ensure lock is removed on script exit.
 */
register_shutdown_function('removeLock');

// Check and create lock to prevent multiple runs.
checkAndCreateLock();


// ---------- Enable Error Reporting ----------
error_reporting(E_ALL);
ini_set('display_errors', 1);

echo "<h1>🟢 Daily Bounces Process initiated...</h1>";

/**
 * Si tu DB col 'email_event_id' es tipo INT y Mautic trae algo como "email.sent12345",
 * necesitamos extraer solo los dígitos (12345).
 */
function parseEventIdAsInt($str) {
    if (is_null($str)) return null;
    // Elimina todo lo que NO sea dígito:
    $digits = preg_replace('/\D/', '', $str);
    if ($digits === '') {
        return null;
    }
    return (int) $digits;
}

/**
 * Refresca el token OAuth de Mautic
 */
function refreshToken() {
    // echo "<p>🔄 Attempting to refresh access token...</p>";

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    // echo "<p>🌐 HTTP Response Code: $httpCode</p>";
    // echo "<pre>📄 Response: $response</pre>";

    if ($httpCode === 200 && strpos($response, 'Authorization successful') !== false) {
        $conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
        $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' LIMIT 1");

        if ($result && $result->num_rows > 0) {
            $GLOBALS['accessToken'] = $result->fetch_assoc()['access_token'];
            // echo "<p>✅ Access token refreshed successfully.</p>";
        } else {
            die("<p>❌ Access token not found in the DB. Exiting...</p>");
        }

        $conn->close();
    } else {
        die("<p>❌ Token refresh failed. HTTP Code: $httpCode.</p>");
    }
}

/**
 * Retorna un array de lead_ids locales que están en leads_segments.segment_id = $segmentId
 * Se usa limit+offset para paginar si hay miles de leads.
 * 
 * (NOTE: This was your original function; we keep it unaltered.)
 */
function getLocalSegmentLeadIds($segmentId, $limit = 1000, $offset = 0) {
    $conn = dbConnect();

    $sql = "SELECT lead_id
            FROM leads_segments
            WHERE segment_id = $segmentId
            ORDER BY lead_id
            LIMIT $limit OFFSET $offset";

    $results = $conn->query($sql);
    $leadIds = [];

    if ($results && $results->num_rows > 0) {
        while ($row = $results->fetch_assoc()) {
            $leadIds[] = $row['lead_id'];
        }
    }

    $conn->close();
    return $leadIds;
}

/*************************************************
 * NEW FUNCTION: getLocalSegmentLeadIdsToProcess
 * Returns leads in segment=$segmentId 
 * where processed_flag=0 
 * up to $limit
 *************************************************/
function getLocalSegmentLeadIdsToProcess($segmentId, $limit = 1000) {
    $conn = dbConnect();

    // Only leads not yet processed (processed_flag=0)
    $sql = "SELECT lead_id
            FROM leads_segments
            WHERE segment_id = $segmentId
              AND processed_flag = 0
            ORDER BY lead_id
            LIMIT $limit";

    $results = $conn->query($sql);
    $leadIds = [];

    if ($results && $results->num_rows > 0) {
        while ($row = $results->fetch_assoc()) {
            $leadIds[] = $row['lead_id'];
        }
    }

    $conn->close();
    return $leadIds;
}

/***********************************************************
 * NEW FUNCTION: markLeadsAsProcessed
 * Sets processed_flag=1 for the given $leadIds in segment_id
 ***********************************************************/
function markLeadsAsProcessed($segmentId, array $leadIds) {
    if (empty($leadIds)) return;
    $conn = dbConnect();

    // Build an IN() list
    $idsList = implode(',', array_map('intval', $leadIds));

    $sql = "UPDATE leads_segments
            SET processed_flag = 1
            WHERE segment_id = $segmentId
              AND lead_id IN ($idsList)";
    $conn->query($sql);

    $conn->close();
}

/**************************************************************
 * NEW FUNCTION: resetProcessedFlagIfAllDone
 * If no leads remain with processed_flag=0, 
 * then reset them all to 0, meaning we can re-process next run.
 **************************************************************/
function resetProcessedFlagIfAllDone($segmentId) {
    $conn = dbConnect();

    // Check if there's any lead with processed_flag=0
    $sqlCheck = "SELECT COUNT(*) AS cnt
                 FROM leads_segments
                 WHERE segment_id = $segmentId
                   AND processed_flag = 0";
    $res = $conn->query($sqlCheck);
    $count = 0;
    if ($res && $row = $res->fetch_assoc()) {
        $count = (int) $row['cnt'];
    }
    if ($count === 0) {
        // Means all leads are processed => reset them for next cycle
        // echo "<p>All leads processed. Resetting processed_flag=0 for entire segment.</p>";
        $sqlReset = "UPDATE leads_segments
                     SET processed_flag = 0
                     WHERE segment_id = $segmentId";
        $conn->query($sqlReset);
    }

    $conn->close();
}

/**
 * Llama a la API de Mautic para obtener la **actividad** completa del contacto
 * con /api/contacts/{leadId}/activity
 */
function fetchContactActivity($leadId) {
    global $accessToken;

    $url = MAUTIC_BASE_URL . "/api/contacts/{$leadId}/activity";

    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        "Authorization: Bearer $accessToken",
        "Content-Type: application/json"
    ]);
    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    if ($httpCode === 200) {
        return json_decode($response, true);
    }
    // En caso de error, retorna array vacío
    return [];
}

/**
 * Procesa el JSON de activity para extraer bounces (lead.donotcontact).
 * Mautic a veces pone reason=2 o "Communication bounced for this channel."
 * Revisamos ambos casos para detectar rebote.
 *
 * Retorna un array con múltiples bounces (posibles) para este lead.
 */
function processActivityForBounces($leadId, $activityData) {
    if (empty($activityData['events']) || !is_array($activityData['events'])) {
        return []; // No hay eventos
    }

    $events = $activityData['events'];

    // Ordenamos eventos cronológicamente (ASC) para "caminar" en orden
    usort($events, function($a, $b) {
        return strtotime($a['timestamp']) <=> strtotime($b['timestamp']);
    });

    $bounces = [];
    $lastEmailSent = null;

    foreach ($events as $event) {
        $eventType = $event['event'] ?? null;
        $timestamp = $event['timestamp'] ?? null;

        // 1) Guardar "último email enviado"
        if ($eventType === 'email.sent') {
            $stat = $event['details']['stat'] ?? [];
            $lastEmailSent = [
                'email_id'  => $stat['email_id'] ?? null,
                'timestamp' => $timestamp,
                'subject'   => $stat['subject'] ?? null,
                'email_name'=> $stat['email_name'] ?? null,
                'list_id'   => $stat['list_id'] ?? null,
                'list_name' => $stat['list_name'] ?? null,
                'eventId'   => $event['eventId'] ?? null, // "email.sentXXXX"
                'label'     => $event['eventLabel']['label'] ?? null,
                'href'      => $event['eventLabel']['href'] ?? null,
            ];
        }
        // 2) Detectar bounce en lead.donotcontact
        elseif ($eventType === 'lead.donotcontact') {
            $dnc = $event['details']['dnc'] ?? [];
            // reason puede ser "2", 2, o "Communication bounced..."
            $rawReason = $dnc['reason'] ?? '';
            $isBounce  = false;

            // (a) si reason es numérico y es 2
            if (is_numeric($rawReason)) {
                if ((int)$rawReason === 2) {
                    $isBounce = true;
                }
            }
            // (b) si reason es string, buscar "bounce"
            else {
                $reasonLower = strtolower($rawReason);
                if (strpos($reasonLower, 'bounce') !== false) {
                    $isBounce = true;
                }
            }

            if ($isBounce) {
                $bounceTimestamp = $timestamp;
                $bounceId        = $dnc['id'] ?? null;
                $bounceComments  = $dnc['comments'] ?? null;
                $bounceDateRaw   = $dnc['date_added'] ?? $bounceTimestamp;

                // Convertir eventId de email y bounce a int
                $emailEventIdInt   = parseEventIdAsInt($lastEmailSent['eventId'] ?? null);
                $bounceEventIdInt  = parseEventIdAsInt($event['eventId'] ?? null);

                $bounceRecord = [
                    'lead_id'            => $leadId,
                    'bounce_id'          => $bounceId,
                    'bounce_date'        => date('Y-m-d H:i:s', strtotime($bounceDateRaw)),
                    'bounce_details'     => json_encode($dnc),
                    'other_bounce_info'  => null,

                    'email_id'           => $lastEmailSent['email_id'] ?? null,
                    'campaign_id'        => null,
                    'email_event_id'     => $emailEventIdInt,
                    'email_label'        => $lastEmailSent['label'] ?? null,
                    'email_href'         => $lastEmailSent['href'] ?? null,
                    'email_sent_timestamp' => (!empty($lastEmailSent['timestamp']))
                        ? date('Y-m-d H:i:s', strtotime($lastEmailSent['timestamp']))
                        : null,
                    'email_subject'      => $lastEmailSent['subject'] ?? null,
                    'email_name'         => $lastEmailSent['email_name'] ?? null,
                    'email_list_id'      => $lastEmailSent['list_id'] ?? null,
                    'email_list_name'    => $lastEmailSent['list_name'] ?? null,

                    'bounce_event_id'    => $bounceEventIdInt,
                    'bounce_reason'      => $rawReason,
                    'bounce_comments'    => $bounceComments,
                ];

                $bounces[] = $bounceRecord;
            }
        }
    }
    return $bounces;
}

/**
 * Inserta/actualiza bounces en la BD, y también imprime en pantalla.
 */
function insertBouncesArray($bounces) {
    foreach ($bounces as $bounce) {
        // 1) Imprimimos en pantalla los datos
        /*
        echo "<div style='border:1px solid #ccc; margin:8px; padding:8px; background:#f9f9f9;'>";
        echo "<h3>Bounce for Lead ID: {$bounce['lead_id']}</h3>";
        echo "<ul>";
        echo "<li><strong>bounce_id:</strong> {$bounce['bounce_id']}</li>";
        echo "<li><strong>bounce_date:</strong> {$bounce['bounce_date']}</li>";
        echo "<li><strong>bounce_details:</strong> {$bounce['bounce_details']}</li>";
        echo "<li><strong>other_bounce_info:</strong> " . ($bounce['other_bounce_info'] ?? '') . "</li>";
        echo "<li><strong>email_id:</strong> " . ($bounce['email_id'] ?? '') . "</li>";
        echo "<li><strong>campaign_id:</strong> " . ($bounce['campaign_id'] ?? '') . "</li>";
        echo "<li><strong>email_event_id:</strong> " . ($bounce['email_event_id'] ?? '') . "</li>";
        echo "<li><strong>email_label:</strong> " . ($bounce['email_label'] ?? '') . "</li>";
        echo "<li><strong>email_href:</strong> " . ($bounce['email_href'] ?? '') . "</li>";
        echo "<li><strong>email_sent_timestamp:</strong> " . ($bounce['email_sent_timestamp'] ?? '') . "</li>";
        echo "<li><strong>email_subject:</strong> " . ($bounce['email_subject'] ?? '') . "</li>";
        echo "<li><strong>email_name:</strong> " . ($bounce['email_name'] ?? '') . "</li>";
        echo "<li><strong>email_list_id:</strong> " . ($bounce['email_list_id'] ?? '') . "</li>";
        echo "<li><strong>email_list_name:</strong> " . ($bounce['email_list_name'] ?? '') . "</li>";
        echo "<li><strong>bounce_event_id:</strong> " . ($bounce['bounce_event_id'] ?? '') . "</li>";
        echo "<li><strong>bounce_reason:</strong> " . ($bounce['bounce_reason'] ?? '') . "</li>";
        echo "<li><strong>bounce_comments:</strong> " . ($bounce['bounce_comments'] ?? '') . "</li>";
        echo "</ul>";
        echo "</div>";
        */

        // 2) Insertar/actualizar en BD
        insertBounceRecord($bounce);
    }
}

/**
 * Simplemente imprime el array de bounce en HTML (ya no usado directamente)
 */
function printBounceRecord($bounce) {
    /*
    echo "<div style='border:1px solid #ccc; margin:8px; padding:8px;'>";
    echo "<h3>Bounce for Lead ID: {$bounce['lead_id']}</h3>";
    echo "<pre>";
    print_r($bounce);
    echo "</pre>";
    echo "</div>";
    */
}

/**
 * Inserta o actualiza un bounce en la tabla local (ON DUPLICATE KEY UPDATE)
 */
function insertBounceRecord($bounce) {
    if (empty($bounce)) return; // nada que insertar

    $conn = dbConnect();

    $stmt = $conn->prepare("
        INSERT INTO mautic_bounce_by_webhook_events (
            lead_id, bounce_id, bounce_date, bounce_details, other_bounce_info,
            email_id, campaign_id, email_event_id, email_label, email_href,
            email_sent_timestamp, email_subject, email_name, email_list_id,
            email_list_name, bounce_event_id, bounce_reason, bounce_comments
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        ON DUPLICATE KEY UPDATE
            bounce_date = VALUES(bounce_date),
            bounce_details = VALUES(bounce_details),
            other_bounce_info = VALUES(other_bounce_info),
            email_id = VALUES(email_id),
            campaign_id = VALUES(campaign_id),
            email_event_id = VALUES(email_event_id),
            email_label = VALUES(email_label),
            email_href = VALUES(email_href),
            email_sent_timestamp = VALUES(email_sent_timestamp),
            email_subject = VALUES(email_subject),
            email_name = VALUES(email_name),
            email_list_id = VALUES(email_list_id),
            email_list_name = VALUES(email_list_name),
            bounce_reason = VALUES(bounce_reason),
            bounce_comments = VALUES(bounce_comments)
    ");

    // Ajustar el orden de bind_param si cambias la estructura
    $stmt->bind_param(
        "iisssiisssssssssss",
        $bounce['lead_id'],
        $bounce['bounce_id'],
        $bounce['bounce_date'],
        $bounce['bounce_details'],
        $bounce['other_bounce_info'],
        $bounce['email_id'],
        $bounce['campaign_id'],
        $bounce['email_event_id'],
        $bounce['email_label'],
        $bounce['email_href'],
        $bounce['email_sent_timestamp'],
        $bounce['email_subject'],
        $bounce['email_name'],
        $bounce['email_list_id'],
        $bounce['email_list_name'],
        $bounce['bounce_event_id'],
        $bounce['bounce_reason'],
        $bounce['bounce_comments']
    );

    if ($stmt->execute()) {
        // echo "✅ Bounce data inserted/updated (Lead ID: {$bounce['lead_id']})<br>";
    } else {
        // echo "❌ Error for Lead ID: {$bounce['lead_id']} - {$stmt->error}<br>";
    }

    $stmt->close();
    $conn->close();
}

/**
 * MAIN EXECUTION
 * - Procesa todo el segmento 7, sin límite de bounces
 *   (inserta/actualiza todos los bounces que existan).
 *
 * CHANGED: We'll now use a processed_flag approach:
 *   1) get leads with processed_flag=0
 *   2) if none => reset processed_flag=0 for next run => break
 *   3) process them => set processed_flag=1 for them
 *   4) repeat until all leads done
 */

// Commenting out old pagination approach. We'll keep lines but comment them:

/*
$SEGMENT_ID    = 7;    // Segmento Bounced
$PAGE_SIZE     = 500;  // Ajusta según tus necesidades
$processedLeads= 0;    // Cuántos leads se revisaron
$totalBounces  = 0;    // Cuántos bounces se insertaron
$page          = 0;

refreshToken();

while (true) {
    $leadIds = getLocalSegmentLeadIds($SEGMENT_ID, $PAGE_SIZE, $page * $PAGE_SIZE);

    if (empty($leadIds)) {
        break;  
    }

    foreach ($leadIds as $leadId) {
        $processedLeads++;

        $activityData = fetchContactActivity($leadId);
        $bounces = processActivityForBounces($leadId, $activityData);

        if (!empty($bounces)) {
            insertBouncesArray($bounces);
            $foundNow = count($bounces);
            $totalBounces += $foundNow;
        }
    }

    $page++;
}
*/

// Instead, new approach with processed_flag and sub-batching:
// =======================
// MAIN EXECUTION
// =======================

$SEGMENT_ID = 7;
$BATCH_SIZE = 1000;  // Maximum records per batch
$SUBCHUNK_SIZE = 50; // Process in sub-chunks of 50
$processedLeads = 0;
$totalBounces = 0;

refreshToken();

// Loop until all leads are processed
while (true) {
    $leadIds = getLocalSegmentLeadIdsToProcess($SEGMENT_ID, $BATCH_SIZE);

    if (empty($leadIds)) {
        resetProcessedFlagIfAllDone($SEGMENT_ID);
        break;
    }

    // Process leads in sub-chunks of 50
    $chunks = array_chunk($leadIds, $SUBCHUNK_SIZE);

    foreach ($chunks as $chunk) {
        foreach ($chunk as $leadId) {
            $processedLeads++;

            $activityData = fetchContactActivity($leadId);
            $bounces = processActivityForBounces($leadId, $activityData);

            if (!empty($bounces)) {
                insertBouncesArray($bounces);
                $totalBounces += count($bounces);
            }
        }

        // Mark sub-chunk of 50 as processed
        markLeadsAsProcessed($SEGMENT_ID, $chunk);
    }
}

// Remove lock after execution.
removeLock();

echo "✅ Done. Processed: $processedLeads leads, $totalBounces bounces inserted/updated.\n";


// optional final echo
// echo "All done: $processedLeads leads processed, $totalBounces bounces inserted/updated.\n";


// We are done
// echo "All done. $processedLeads leads were processed in this run. $totalBounces bounces found/updated.";

?>
