<?php
// Include configuration and utility functions
require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/../includes/utils.php';

// *****************************************
// * Email Inventory Fetch & Save         *
// *****************************************

function fetchAndSaveEmails() {
    global $accessToken;
    $start = 0;
    $limit = 100; // Optimized batch size
    $totalFetched = 0;
    $maxAttempts = 3; // Limit retries to prevent system overload

    echo "🟢 Process initiated...<br>";
    echo "✅ cURL is enabled.<br>";
    echo "🚀 Fetching emails...<br>";

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/emails?publishedOnly=false&limit=$limit&start=$start";
        logMessage("📦 Fetching emails from URL: $url");
        echo "📦 Fetching emails from URL: $url<br>";

        $attempts = 0;
        $success = false;

        while ($attempts < $maxAttempts && !$success) {
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
            $response = curl_exec($ch);
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            if ($httpCode === 200) {
                $data = json_decode($response, true);
                $emails = $data['emails'] ?? [];
                $fetchedCount = count($emails);

                if ($fetchedCount > 0) {
                    saveEmailInventory($emails);
                    $totalFetched += $fetchedCount;
                    logMessage("📨 Fetched $fetchedCount emails. Total fetched so far: $totalFetched");
                    echo "📨 Fetched $fetchedCount emails. Total fetched so far: $totalFetched<br>";
                    $start += $limit;
                } else {
                    logMessage("🛑 No more emails to fetch. Ending process.");
                    echo "🛑 No more emails to fetch. Ending process.<br>";
                    fetchEmailDetails();
                    return;
                }

                $success = true;
            } else {
                $attempts++;
                logMessage("❌ Failed to fetch emails. HTTP Code: $httpCode. Attempt $attempts/$maxAttempts");
                echo "❌ Failed to fetch emails. HTTP Code: $httpCode. Attempt $attempts/$maxAttempts<br>";
                sleep(2);
            }
        }

        if (!$success) {
            logMessage("🚫 Max retry attempts reached. Stopping fetch process.");
            echo "🚫 Max retry attempts reached. Stopping fetch process.<br>";
            return;
        }
    }
}

function saveEmailInventory($emails) {
    $conn = dbConnect();
    $stmt = $conn->prepare("REPLACE INTO emma_mason_email_list (email_id, name) VALUES (?, ?)");

    foreach ($emails as $email) {
        $stmt->bind_param("is", $email['id'], $email['name']);
        if ($stmt->execute()) {
            logMessage("✅ Saved Email ID: {$email['id']} - {$email['name']}");
            echo "✅ Saved Email ID: {$email['id']} - {$email['name']}<br>";
        } else {
            logMessage("❌ Failed to save Email ID: {$email['id']} - Error: " . $stmt->error);
            echo "❌ Failed to save Email ID: {$email['id']} - Error: " . $stmt->error . "<br>";
        }
    }

    $stmt->close();
    $conn->close();
}

function fetchEmailDetails() {
    global $accessToken;
    $conn = dbConnect();
    $result = $conn->query("SELECT email_id FROM emma_mason_email_list");

    while ($row = $result->fetch_assoc()) {
        $emailId = $row['email_id'];
        $url = MAUTIC_BASE_URL . "/api/emails/$emailId";
        logMessage("📥 Fetching details for Email ID: $emailId");
        echo "📥 Fetching details for Email ID: $emailId<br>";

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode === 200) {
            $emailDetails = json_decode($response, true)['email'];
            saveEmailDetails($emailDetails);
        } else {
            logMessage("❌ Failed to fetch details for Email ID: $emailId. HTTP Code: $httpCode");
            echo "❌ Failed to fetch details for Email ID: $emailId. HTTP Code: $httpCode<br>";
        }
    }

    $conn->close();
}

function saveEmailDetails($email) {
    $conn = dbConnect();
    $stmt = $conn->prepare("REPLACE INTO emma_mason_email_details (email_id, name, subject, from_address, from_name, is_published, publish_up, publish_down) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");

    $publishUp = convertToMySQLDateTime($email['publishUp']);
    $publishDown = convertToMySQLDateTime($email['publishDown']);
    $isPublished = isset($email['isPublished']) && $email['isPublished'] ? 1 : 0;

    $stmt->bind_param("isssssss",
        $email['id'],
        $email['name'],
        $email['subject'],
        $email['fromAddress'],
        $email['fromName'],
        $isPublished,
        $publishUp,
        $publishDown
    );

    if ($stmt->execute()) {
        logMessage("✅ Detailed Email ID {$email['id']} saved.");
        echo "✅ Detailed Email ID {$email['id']} saved.<br>";
    } else {
        logMessage("❌ Failed to save details for Email ID {$email['id']}: " . $stmt->error);
        echo "❌ Failed to save details for Email ID {$email['id']}: " . $stmt->error . "<br>";
    }

    $stmt->close();
    $conn->close();
}

fetchAndSaveEmails();
?>
