<?php
// Include configuration and utility functions
require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/../includes/utils.php';

// *****************************************
// * Fetch and Save Tags to the Database   *
// *****************************************

function fetchAndSaveTags() {
    global $accessToken;
    $start = 0;
    $limit = 100;
    $totalFetched = 0;
    $maxAttempts = 3;

    echo "🟢 Fetching tags initiated...<br>";
    logMessage("🟢 Fetching tags initiated...");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/tags?limit=$limit&start=$start";
        logMessage("📦 Fetching tags from URL: $url");
        echo "📦 Fetching tags from URL: $url<br>";

        $attempts = 0;
        $success = false;

        while ($attempts < $maxAttempts && !$success) {
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
            $response = curl_exec($ch);
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            if ($httpCode === 200) {
                $data = json_decode($response, true);
                $tags = $data['tags'] ?? [];
                $fetchedCount = count($tags);

                if ($fetchedCount > 0) {
                    saveTags($tags);
                    $totalFetched += $fetchedCount;
                    logMessage("📨 Fetched $fetchedCount tags. Total fetched so far: $totalFetched");
                    echo "📨 Fetched $fetchedCount tags. Total fetched so far: $totalFetched<br>";
                    $start += $limit;
                } else {
                    logMessage("🛑 No more tags to fetch. Ending process.");
                    echo "🛑 No more tags to fetch. Ending process.<br>";
                    return;
                }

                $success = true;
            } else {
                $attempts++;
                logMessage("❌ Failed to fetch tags. HTTP Code: $httpCode. Attempt $attempts/$maxAttempts");
                echo "❌ Failed to fetch tags. HTTP Code: $httpCode. Attempt $attempts/$maxAttempts<br>";
                sleep(2);
            }
        }

        if (!$success) {
            logMessage("🚫 Max retry attempts reached. Stopping fetch process.");
            echo "🚫 Max retry attempts reached. Stopping fetch process.<br>";
            return;
        }
    }
}

function saveTags($tags) {
    $conn = dbConnect();

    foreach ($tags as $tag) {
        $tagId = $tag['id'];
        $name = $tag['tag'];
        $description = $tag['description'] ?? null;
        $createdAt = convertToMySQLDateTime($tag['dateAdded'] ?? null);

        $stmt = $conn->prepare("REPLACE INTO emma_mason_tags (tag_id, name, description, created_at) VALUES (?, ?, ?, ?)");
        $stmt->bind_param("isss", $tagId, $name, $description, $createdAt);

        if ($stmt->execute()) {
            logMessage("✅ Tag ID $tagId saved.");
            echo "✅ Tag ID $tagId saved.<br>";
        } else {
            logMessage("❌ Failed to save Tag ID $tagId: " . $stmt->error);
            echo "❌ Failed to save Tag ID $tagId: " . $stmt->error . "<br>";
        }

        $stmt->close();
    }

    $conn->close();
    logMessage("✅ All fetched tags have been saved.");
}

// Run the function to fetch and save tags
fetchAndSaveTags();
?>
