<?php
require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// Conectar a la base de datos
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);

if ($conn->connect_error) {
    die("❌ Error de conexión a la base de datos: " . $conn->connect_error);
}

// 1️⃣ PRIMER PROCESO: Truncar e insertar email_id en email_click_analysis_report
function insertEmailIds($conn) {
    logMessage("Proceso 1: Insertar Email IDs");

    // Obtener email_id de email_activity_per_day
    $sqlEmailActivity = "SELECT DISTINCT email_id FROM email_activity_per_day";
    $resultEmailActivity = $conn->query($sqlEmailActivity);

    if (!$resultEmailActivity) {
        logMessage("❌ Error en la consulta de email_activity_per_day: " . $conn->error);
        return [];
    }

    $emailIds = [];
    while ($row = $resultEmailActivity->fetch_assoc()) {
        $emailIds[] = $row['email_id'];
    }

    // Si no hay datos, salir
    if (empty($emailIds)) {
        logMessage("⚠️ No se encontraron email_id en email_activity_per_day.");
        return [];
    }

    // Obtener los email_id de eventos de tipo 'email.send'
    $sqlCampaignEvents = "SELECT DISTINCT channel_id AS email_id FROM emma_mason_campaign_events WHERE type = 'email.send'";
    $resultCampaignEvents = $conn->query($sqlCampaignEvents);

    if (!$resultCampaignEvents) {
        logMessage("❌ Error en la consulta de emma_mason_campaign_events: " . $conn->error);
        return [];
    }

    $campaignEmailIds = [];
    while ($row = $resultCampaignEvents->fetch_assoc()) {
        $campaignEmailIds[] = $row['email_id'];
    }

    // Filtrar email_id que no están en los eventos 'email.send'
    $missingEmails = array_diff($emailIds, $campaignEmailIds);

    // TRUNCATE tabla antes de insertar
    $truncateSQL = "TRUNCATE TABLE email_click_analysis_report;";
    if ($conn->query($truncateSQL) === TRUE) {
        logMessage("✅ Tabla email_click_analysis_report truncada correctamente.");
    } else {
        logMessage("❌ Error al truncar la tabla: " . $conn->error);
        return [];
    }

    // Insertar email_id en la tabla
    $successCount = 0;
    foreach ($missingEmails as $emailId) {
        $insertSQL = "INSERT INTO email_click_analysis_report (email_id) VALUES ($emailId);";
        if ($conn->query($insertSQL) === TRUE) {
            $successCount++;
        }
    }

    logMessage("✅ Se insertaron $successCount email_id en email_click_analysis_report.");
    return $missingEmails;
}

// 2️⃣ SEGUNDO PROCESO: Obtener la fecha de envío más antigua (MIN(date_sent)) para cada email_id
function getOldestSentDates($conn, $emailIds) {
    logMessage("Proceso 2: Buscar Fechas de Envío Más Antiguas");

    if (empty($emailIds)) {
        logMessage("⚠️ No hay email_id para procesar.");
        return [];
    }

    $missingEmailsList = implode(",", array_map('intval', $emailIds));
    $sqlDateSent = "
        SELECT email_id, MIN(date_sent) AS oldest_date_sent
        FROM email_activity_per_day
        WHERE email_id IN ($missingEmailsList)
        GROUP BY email_id;
    ";

    $resultDateSent = $conn->query($sqlDateSent);

    if (!$resultDateSent) {
        logMessage("❌ Error en la consulta de date_sent: " . $conn->error);
        return [];
    }

    $emailSentDates = [];
    while ($row = $resultDateSent->fetch_assoc()) {
        $emailSentDates[$row['email_id']] = $row['oldest_date_sent'];
    }

    logMessage("✅ Se encontraron fechas de envío antiguas para " . count($emailSentDates) . " email_id.");

    return $emailSentDates;
}


// 3️⃣ TERCER PROCESO: Actualizar sent_date en email_click_analysis_report
function updateSentDates($conn, $emailSentDates) {
    logMessage("Proceso 3: Actualizar Fechas en la Base de Datos");

    if (empty($emailSentDates)) {
        logMessage("⚠️ No hay fechas de envío para actualizar.");
        return;
    }

    $successCount = 0;
    foreach ($emailSentDates as $emailId => $sentDate) {
        $sentDateValue = $sentDate ? "'$sentDate'" : "NULL";
        $updateSQL = "UPDATE email_click_analysis_report SET sent_date = $sentDateValue WHERE email_id = $emailId;";

        if ($conn->query($updateSQL) === TRUE) {
            $successCount++;
        } else {
            logMessage("❌ Error al actualizar sent_date para email_id $emailId: " . $conn->error);
        }
    }

    logMessage("✅ Se actualizaron $successCount registros con la fecha de envío más antigua en email_click_analysis_report.");
}

// Verificar si la columna `sent_date` existe antes de ejecutar la consulta
function checkSentDateColumn($conn) {
    $result = $conn->query("SHOW COLUMNS FROM email_click_analysis_report LIKE 'sent_date'");
    return ($result && $result->num_rows > 0);
}


// 4️⃣ PROCESO: Contar clics por días después del envío e insertar en la BD
function countClicksOnSentDateAndInsert($conn) {
    logMessage("Proceso 4: Contar Clics por Día después del Envío e Insertar en BD");

    // Obtener email_id y sent_date de email_click_analysis_report
    $sqlGetEmails = "
        SELECT email_id, sent_date 
        FROM email_click_analysis_report 
        WHERE sent_date IS NOT NULL
    ";

    $resultEmails = $conn->query($sqlGetEmails);

    if (!$resultEmails) {
        logMessage("❌ Error en la consulta de email_click_analysis_report: " . $conn->error);
        return;
    }

    while ($row = $resultEmails->fetch_assoc()) {
        $emailId = intval($row['email_id']);
        $sentDate = $row['sent_date'];

        // Consulta para contar clics por días después del envío
        $sqlCountClicks = "
            SELECT 
                DATEDIFF(click_date, '$sentDate') AS days_after_sent,
                COUNT(*) AS total_clicks,
                COUNT(DISTINCT lead_id) AS unique_leads
            FROM parsed_click_data
            WHERE email_id = $emailId
            AND DATE(click_date) >= '$sentDate'
            GROUP BY days_after_sent
        ";

        $resultClicks = $conn->query($sqlCountClicks);

        // Inicializar contadores en 0
        $clicksByDays = array_fill_keys([
            'total_clicks_day_0', 'total_clicks_day_1', 'total_clicks_day_2', 'total_clicks_day_3',
            'total_clicks_day_4', 'total_clicks_day_5', 'total_clicks_day_6', 'total_clicks_day_7_or_more',
            'total_historical_clicks'
        ], 0);

        $uniqueClicksByDays = array_fill_keys([
            'unique_clicks_day_0', 'unique_clicks_day_1', 'unique_clicks_day_2', 'unique_clicks_day_3',
            'unique_clicks_day_4', 'unique_clicks_day_5', 'unique_clicks_day_6', 'unique_clicks_day_7_or_more',
            'unique_historical_clicks'
        ], 0);

        if ($resultClicks) {
            while ($clickRow = $resultClicks->fetch_assoc()) {
                $daysAfterSent = intval($clickRow['days_after_sent']);
                $totalClicks = intval($clickRow['total_clicks']);
                $uniqueLeads = intval($clickRow['unique_leads']);

                if ($daysAfterSent == 0) {
                    $clicksByDays['total_clicks_day_0'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_0'] += $uniqueLeads;
                } elseif ($daysAfterSent == 1) {
                    $clicksByDays['total_clicks_day_1'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_1'] += $uniqueLeads;
                } elseif ($daysAfterSent == 2) {
                    $clicksByDays['total_clicks_day_2'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_2'] += $uniqueLeads;
                } elseif ($daysAfterSent == 3) {
                    $clicksByDays['total_clicks_day_3'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_3'] += $uniqueLeads;
                } elseif ($daysAfterSent == 4) {
                    $clicksByDays['total_clicks_day_4'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_4'] += $uniqueLeads;
                } elseif ($daysAfterSent == 5) {
                    $clicksByDays['total_clicks_day_5'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_5'] += $uniqueLeads;
                } elseif ($daysAfterSent == 6) {
                    $clicksByDays['total_clicks_day_6'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_6'] += $uniqueLeads;
                } elseif ($daysAfterSent >= 7) {
                    $clicksByDays['total_clicks_day_7_or_more'] += $totalClicks;
                    $uniqueClicksByDays['unique_clicks_day_7_or_more'] += $uniqueLeads;
                }

                // Sumar los totales históricos
                $clicksByDays['total_historical_clicks'] += $totalClicks;
                $uniqueClicksByDays['unique_historical_clicks'] += $uniqueLeads;
            }
        }

        // 🔥 **INSERT correcto**
        $insertSQL = "
            UPDATE email_click_analysis_report SET
                total_clicks_day_0 = {$clicksByDays['total_clicks_day_0']},
                total_clicks_day_1 = {$clicksByDays['total_clicks_day_1']},
                total_clicks_day_2 = {$clicksByDays['total_clicks_day_2']},
                total_clicks_day_3 = {$clicksByDays['total_clicks_day_3']},
                total_clicks_day_4 = {$clicksByDays['total_clicks_day_4']},
                total_clicks_day_5 = {$clicksByDays['total_clicks_day_5']},
                total_clicks_day_6 = {$clicksByDays['total_clicks_day_6']},
                total_clicks_day_7_or_more = {$clicksByDays['total_clicks_day_7_or_more']},
                total_historical_clicks = {$clicksByDays['total_historical_clicks']},
                unique_clicks_day_0 = {$uniqueClicksByDays['unique_clicks_day_0']},
                unique_clicks_day_1 = {$uniqueClicksByDays['unique_clicks_day_1']},
                unique_clicks_day_2 = {$uniqueClicksByDays['unique_clicks_day_2']},
                unique_clicks_day_3 = {$uniqueClicksByDays['unique_clicks_day_3']},
                unique_clicks_day_4 = {$uniqueClicksByDays['unique_clicks_day_4']},
                unique_clicks_day_5 = {$uniqueClicksByDays['unique_clicks_day_5']},
                unique_clicks_day_6 = {$uniqueClicksByDays['unique_clicks_day_6']},
                unique_clicks_day_7_or_more = {$uniqueClicksByDays['unique_clicks_day_7_or_more']},
                unique_historical_clicks = {$uniqueClicksByDays['unique_historical_clicks']}
            WHERE email_id = $emailId;
        ";

        if ($conn->query($insertSQL) === TRUE) {
            logMessage("✅ Datos actualizados para email_id: $emailId.");
        } else {
            logMessage("❌ Error al actualizar email_id $emailId: " . $conn->error);
        }
    }
}


// 5️⃣ PROCESO: Insertar el nombre del email en email_click_analysis_report
function updateEmailNames($conn) {
    logMessage("Proceso 5: Actualizar Nombres de Emails");

    // Obtener email_id y nombres de la tabla emma_mason_email_list
    $sqlFetchEmails = "
        SELECT email_id, name 
        FROM emma_mason_email_list
    ";
    $resultEmails = $conn->query($sqlFetchEmails);

    if (!$resultEmails) {
        logMessage("❌ Error en la consulta de emma_mason_email_list: " . $conn->error);
        return;
    }

    $successCount = 0;

    while ($row = $resultEmails->fetch_assoc()) {
        $emailId = intval($row['email_id']);
        $emailName = $conn->real_escape_string($row['name']);

        // Actualizar el nombre del email en la tabla email_click_analysis_report
        $updateSQL = "
            UPDATE email_click_analysis_report 
            SET email_name = '$emailName' 
            WHERE email_id = $emailId;
        ";

        if ($conn->query($updateSQL) === TRUE) {
            $successCount++;
        } else {
            logMessage("❌ Error al actualizar email_name para email_id $emailId: " . $conn->error);
        }
    }

    logMessage("✅ Se actualizaron $successCount registros con los nombres de emails.");
}


// 1️⃣ PRIMER PROCESO: Insertar Email IDs en email_click_analysis_report
logMessage("🔹 Iniciando Proceso 1: Insertar Email IDs");
$missingEmails = insertEmailIds($conn);

// 2️⃣ SEGUNDO PROCESO: Obtener la fecha de envío más antigua
logMessage("🔹 Iniciando Proceso 2: Obtener Fechas de Envío Más Antiguas");
$emailSentDates = getOldestSentDates($conn, $missingEmails);

// 3️⃣ TERCER PROCESO: Actualizar las fechas de envío en email_click_analysis_report
logMessage("🔹 Iniciando Proceso 3: Actualizar Fechas en BD");
updateSentDates($conn, $emailSentDates);

// 4️⃣ CUARTO PROCESO: Contar clics por días después del envío e insertar en BD
logMessage("🔹 Iniciando Proceso 4: Contar Clics y Guardar en BD");
countClicksOnSentDateAndInsert($conn);

// 5️⃣ QUINTO PROCESO: Insertar el nombre del email en email_click_analysis_report
logMessage("🔹 Iniciando Proceso 5: Actualizar Nombres de Emails");
updateEmailNames($conn);

// Cerrar conexión
$conn->close();
logMessage("✅ TODOS LOS PROCESOS SE EJECUTARON EXITOSAMENTE");
