<?php
require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/../includes/utils.php';

// Connect to the database
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);

// Check connection
if ($conn->connect_error) {
    logMessage("❌ [DB] Connection failed: " . $conn->connect_error);
    die("Connection failed: " . $conn->connect_error);
}

logMessage("🚀 [PROCESS] Starting processing of parsed data.");

// Batch size
$batchSize = 1000;

// ======================== Part 1: Populate `parsed_click_data` ========================

$lastProcessedClickIdResult = $conn->query("SELECT MAX(original_id) AS last_id FROM parsed_click_data");
$lastProcessedClickId = $lastProcessedClickIdResult->fetch_assoc()['last_id'] ?? 0;

logMessage("🔎 [CLICK DATA] Last processed ID: $lastProcessedClickId");

while (true) {
    $sql = "SELECT id, query, date_hit FROM mautic_page_hits WHERE query IS NOT NULL AND id > $lastProcessedClickId ORDER BY id ASC LIMIT $batchSize";
    $result = $conn->query($sql);

    if ($result->num_rows > 0) {
        $values = [];
        while ($row = $result->fetch_assoc()) {
            $queryData = $row['query'];
            $parsedData = @unserialize($queryData);

            // Ensure valid serialization
            if ($parsedData !== false && is_array($parsedData)) {
                $originalId = $row['id'];
                $clickedUrl = $parsedData['page_url'] ?? null;
                $emailId = $parsedData['ct']['email'] ?? null;
                $campaignId = $parsedData['utm_campaign'] ?? null; // Fix for utm_campaign
                $leadId = isset($parsedData['ct']['lead']) ? intval($parsedData['ct']['lead']) : null;
                $statId = $parsedData['ct']['stat'] ?? null;
                $utmSource = $parsedData['utm_source'] ?? null;
                $utmMedium = $parsedData['utm_medium'] ?? null;
                $utmContent = $parsedData['utm_content'] ?? null;
                $eventId = $parsedData['ct']['source'][1] ?? null; // Example for nested values
                $jsonString = json_encode($parsedData, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);

                $values[] = sprintf(
                    "(%d, %s, 'click', %s, %s, %s, %s, %s, %s, %s, %s, %s, '%s')",
                    $originalId,
                    $row['date_hit'] !== null ? "'" . $conn->real_escape_string($row['date_hit']) . "'" : "NULL",
                    $clickedUrl !== null ? "'" . $conn->real_escape_string($clickedUrl) . "'" : "NULL",
                    $emailId !== null ? "'" . $conn->real_escape_string($emailId) . "'" : "NULL",
                    $campaignId !== null ? "'" . $conn->real_escape_string($campaignId) . "'" : "NULL",
                    $leadId !== null ? "'" . $conn->real_escape_string($leadId) . "'" : "NULL",
                    $utmSource !== null ? "'" . $conn->real_escape_string($utmSource) . "'" : "NULL",
                    $utmMedium !== null ? "'" . $conn->real_escape_string($utmMedium) . "'" : "NULL",
                    $utmContent !== null ? "'" . $conn->real_escape_string($utmContent) . "'" : "NULL",
                    $statId !== null ? "'" . $conn->real_escape_string($statId) . "'" : "NULL",
                    $eventId !== null ? "'" . $conn->real_escape_string($eventId) . "'" : "NULL",
                    $conn->real_escape_string($jsonString)
                );

                $lastProcessedClickId = $originalId;
            } else {
                logMessage("⚠️ [CLICK DATA] Skipped record ID $row[id]: Invalid or non-array query data.");
            }
        }

        if (!empty($values)) {
            $insertSql = "
                INSERT INTO parsed_click_data (
                    original_id, click_date, event, clicked_url, email_id, campaign_id, lead_id, 
                    utm_source, utm_medium, utm_content, stat_id, event_id, json_data
                ) VALUES " . implode(", ", $values);

            if ($conn->query($insertSql) === TRUE) {
                logMessage("✅ [CLICK DATA] Inserted " . count($values) . " records. Last processed ID: $lastProcessedClickId");
            } else {
                logMessage("❌ [CLICK DATA] Error inserting records: " . $conn->error);
                break;
            }
        }
    } else {
        logMessage("🛑 [CLICK DATA] All records processed up to ID: $lastProcessedClickId");
        break;
    }
}

// ======================== Part 2: Populate `parsed_json_details` ========================

function extractNestedData($data, $originalId, &$insertValues, $conn, $parentKey = '') {
    foreach ($data as $key => $value) {
        $fullKey = $parentKey ? $parentKey . '.' . $key : $key;
        if (is_array($value)) {
            extractNestedData($value, $originalId, $insertValues, $conn, $fullKey);
        } else {
            $insertValues[] = sprintf(
                "(%d, '%s', '%s')",
                $originalId,
                $conn->real_escape_string($fullKey),
                $conn->real_escape_string((string)$value)
            );
        }
    }
}

$lastProcessedJsonIdResult = $conn->query("SELECT MAX(original_id) AS last_id FROM parsed_json_details");
$lastProcessedJsonId = $lastProcessedJsonIdResult->fetch_assoc()['last_id'] ?? 0;

logMessage("🔎 [JSON DETAILS] Last processed ID: $lastProcessedJsonId");

while (true) {
    $sql = "SELECT id, query FROM mautic_page_hits WHERE query IS NOT NULL AND id > $lastProcessedJsonId ORDER BY id ASC LIMIT $batchSize";
    $result = $conn->query($sql);

    if ($result->num_rows > 0) {
        $insertValues = [];
        while ($row = $result->fetch_assoc()) {
            $originalId = $row['id'];
            $queryData = @unserialize($row['query']);

            if ($queryData !== false && is_array($queryData)) {
                extractNestedData($queryData, $originalId, $insertValues, $conn);
                $lastProcessedJsonId = $originalId;
            } else {
                logMessage("⚠️ [JSON DETAILS] Skipped record ID $originalId: Invalid or non-array query data.");
            }
        }

        if (!empty($insertValues)) {
            $insertSql = "
                INSERT INTO parsed_json_details (original_id, key_name, key_value)
                VALUES " . implode(", ", $insertValues);

            if ($conn->query($insertSql) === TRUE) {
                logMessage("✅ [JSON DETAILS] Inserted " . count($insertValues) . " records. Last processed ID: $lastProcessedJsonId");
            } else {
                logMessage("❌ [JSON DETAILS] Error inserting records: " . $conn->error);
                break;
            }
        }
    } else {
        logMessage("🛑 [JSON DETAILS] All records processed up to ID: $lastProcessedJsonId");
        break;
    }
}

// ======================== Part 3: Populate `clicks_per_day_email_campaign` ========================

logMessage("🔄 [AGGREGATION] Refreshing clicks_per_day_email_campaign table.");

// Populate the table with aggregated data, including unique lead counts
$populateTableSQL = "
    REPLACE INTO clicks_per_day_email_campaign (click_date, email_id, campaign_id, total_clicks, total_different_leads_that_clicked)
    SELECT 
        DATE(click_date) AS click_date,
        email_id,
        campaign_id,
        COUNT(*) AS total_clicks,
        COUNT(DISTINCT lead_id) AS total_different_leads_that_clicked
    FROM 
        parsed_click_data
    WHERE 
        email_id IS NOT NULL AND campaign_id IS NOT NULL AND lead_id IS NOT NULL
    GROUP BY 
        DATE(click_date), email_id, campaign_id";
if ($conn->query($populateTableSQL) === TRUE) {
    logMessage("✅ [AGGREGATION] Table `clicks_per_day_email_campaign` updated successfully.");
} else {
    logMessage("❌ [AGGREGATION] Error updating table: " . $conn->error);
}

logMessage("🎉 [PROCESS] Completed processing and aggregation.");

$conn->close();

